class Roqua::Scheduling::Schedule
  attr_reader :tasks

  BEGINNING_OF_EVERY_DAY = proc { Time.now.beginning_of_day + 1.day }
  BEGINNING_OF_EVERY_HOUR = proc { Time.now.beginning_of_hour + 1.hour }

  def initialize
    @tasks = {}
  end

  def add_task(name, options, &block)
    @tasks[name] = Roqua::Scheduling::Task.new(name, options, block)
  end

  def self.setup
    @schedule = Roqua::Scheduling::Schedule.new.tap do |new_schedule|
      yield(new_schedule)
      new_schedule.initialize_cronjob_table
    end
  end

  def self.current_schedule
    @schedule ||= Roqua::Scheduling::Schedule.new
  end

  def initialize_cronjob_table
    Roqua::Scheduling::CronJob.where('name NOT IN (?)', tasks.values.map(&:name)).delete_all

    tasks.each_value do |task|
      cron_job = Roqua::Scheduling::CronJob.find_or_initialize_by(name: task.name)
      cron_job.update next_run_at: task.next_run_at unless cron_job.persisted?
    end
  end
end
